﻿using System;

namespace Microsoft.Xna.Framework.Graphics
{
    public unsafe static class DefaultColorProcessors
    {
        private static byte ApplyAlpha(byte color, byte alpha)
        {
            var fc = color / 255.0f;
            var fa = alpha / 255.0f;
            var fr = (int)(255.0f * fc * fa);

            return (byte)MathHelper.Clamp(fr, byte.MinValue, byte.MaxValue);
        }

        /// <summary>
        /// Zeroes RGB of pixels having zero alpha(standard XNA behavior)
        /// </summary>
        public static readonly Action<byte[]> ZeroTransparentPixels = data =>
        {
            fixed (byte* b = &data[0])
            {
                byte* ptr = b;
                for (var i = 0; i < data.Length; i += 4, ptr += 4)
                {
                    if (ptr[3] == 0)
                    {
                        ptr[0] = ptr[1] = ptr[2] = 0;
                    }
                }
            }
        };

        /// <summary>
        /// Premultiplies RGB of pixels by its alpha
        /// </summary>
        public static readonly Action<byte[]> PremultiplyAlpha = data =>
        {
            fixed (byte* b = &data[0])
            {
                byte* ptr = b;
                for (var i = 0; i < data.Length; i += 4, ptr += 4)
                {
                    var a = ptr[3];
                    ptr[0] = ApplyAlpha(ptr[0], a);
                    ptr[1] = ApplyAlpha(ptr[1], a);
                    ptr[2] = ApplyAlpha(ptr[2], a);
                }
            }
        };
    }
}
